import {LabLayoutComponentModel, LayoutComponentPosition} from "../LabLayoutComponentStore";
import {CanvasRenderer} from "./CanvasRenderer";

export class LayoutEditViewRenderer extends CanvasRenderer {
    constructor() {
        super();
    }

    public deleteSelectedComponent(): void {
        if (!this.selectedComponent) return;
        if (this.selectedComponent.component_type === 'status_legend') return; // cannot delete
        this.layout.deleteComponent(this.selectedComponent);
        this.setSelectedComponent(undefined);
    }

    protected handleDraw(): void {
        // Use the view model to render the positions
        this.layout.current_ordered_component.forEach(c => {
            switch (c.component_type) {
                case "machine_status_dot":
                    this.drawDetailedMachineDot(c);
                    break;
                case "image":
                    this.drawImage(c);
                    break;
                case "alert_group":
                    this.drawDetailedAlertGroup(c);
                    break;
                case "status_legend":
                    this.drawStatusLegend(c);
                    break;
            }
        });
        this.drawSelectedComponentBoundaries();
    }

    protected afterSetLayout(): void {
        // Nothing to do here in edit mode
    }

    private drawDetailedMachineDot(c: LabLayoutComponentModel): void {
        const machineName = c.name;
        const machineDuration = 'XX:XX:XX';
        this.drawMachineInfo(c, machineName, machineDuration, LayoutEditViewRenderer.MACHINE_STATUS_DOT_COLOR);
    }

    private drawDetailedAlertGroup(c: LabLayoutComponentModel): void {
        this.drawAlertGroup(c, {fillColor: CanvasRenderer.COLOR_ALERT_OFF});
        this.drawText(c.name, {x: c.position.x, y: c.position.y - 5})
    }

    private legendEnabled(): boolean {
        const comp = this.layout.current_ordered_component.find(c => c.component_type === 'status_legend');

        if(!comp || !comp.options || !JSON.parse(comp.options).show){
            return false;
        }
        return true;
    }

    private drawStatusLegend(c: LabLayoutComponentModel): void {
        if (!this.legendEnabled()) {
            return; // do not draw
        }

        const descriptionText = 'All statuses will be displayed here';

        const scalingRatio = this.scaleRatio;

        // HORIZONTAL SPACING
        const horizontalPadding = (10 * scalingRatio.x);
        const insetPadding = (20 * scalingRatio.x);

        // VERTICAL SPACING
        const verticalTitleMargin = (40 * scalingRatio.y);
        const verticalPadding = (30 * scalingRatio.y);

        const rectWidth = c.width * scalingRatio.x;
        const rectHeight = c.height * scalingRatio.y;

        const legendTitleTextPos = {
            x: c.position.x + horizontalPadding,
            y: c.position.y + verticalPadding
        };

        const descriptionTextPos = {
            x: c.position.x + horizontalPadding + insetPadding,
            y: c.position.y + verticalPadding + verticalTitleMargin
        };

        this.drawRectangle(c.position, rectWidth, rectHeight, { fillColor: 'rgba(255, 255, 255, 0.5)', strokeColor: 'black', lineWidth: 2 });
        this.drawStatusLegendTitleText(c.name, legendTitleTextPos);

        this.drawDescriptionText(descriptionText, descriptionTextPos);
    }

    private drawDescriptionText(text: string, pos: LayoutComponentPosition): void {
        const scalingRatio = this.scaleRatio;
        const fontSize = (16 * scalingRatio.x) + 'px';
        this.ctx.font = '600 ' + fontSize + ' Segoe UI';
        this.ctx.fillStyle = "black";
        this.ctx.textAlign = "start";
        this.ctx.fillText(text, pos.x, pos.y);
    }

    private drawSelectedComponentBoundaries(): void {
        const c = this.selectedComponent;
        if (!c) return;

        if (c.component_type === 'status_legend') {
            const scalingRatio = this.scaleRatio;
            this.drawRectangle(c.position, c.width * scalingRatio.x, c.height * scalingRatio.y, LayoutEditViewRenderer.SELECTED_COMPONENT_BORDER_STYLE);
            return; // cannot resize, don't draw line
        }

        this.drawRectangle(c.position, c.width, c.height, LayoutEditViewRenderer.SELECTED_COMPONENT_BORDER_STYLE);

        this.drawLine({x: c.x + c.width - 15, y: c.y + c.height}, {
            x: c.x + c.width,
            y: c.y + c.height - 15
        }, LayoutEditViewRenderer.SELECTED_COMPONENT_BORDER_STYLE);
    }

    private static MACHINE_STATUS_DOT_COLOR = "black";
    private static SELECTED_COMPONENT_BORDER_STYLE = {strokeColor: "red", lineWidth: 1};

    //// Canvas Action Handlers

    public handleMouseMove(e: any): void {
        if (e.touches) e.preventDefault();
        const point = this.eventInputPosition(e);
        const c = this.selectedComponent;
        if (c && (this.state === 'mouse_down' || this.state === 'dragging')) {
            if (this.state === 'mouse_down') {
                if (point.x > c.x && point.x < c.x + c.width &&
                    point.y > c.y && point.y < c.y + c.height) {
                    this.state = 'dragging';
                }
            } else if (this.state === 'dragging') {
                c.x = c.x + (point.x - this.lastMousePosition.x);
                c.y = c.y + (point.y - this.lastMousePosition.y);
                this.lastMousePosition = point;
            }
        } else if (c && this.state === 'resizing') {
            const x = c.x + c.width;
            const xOffset = point.x - x;
            c.width += xOffset;
            c.height += xOffset / c.aspect_ratio;
        }
    }

    public handleMouseUp(e: any): void {
        if (this.state === 'saving') return;
        if (this.state === 'mouse_down') {
            const point = this.eventInputPosition(e);

            let components = this.layout.current_ordered_component.reverse();
            if(!this.legendEnabled()){
                components = components.filter(c => c.component_type !== 'status_legend');
            }

            const comp = components.find(c => {
                return point.x > c.x && point.x < c.x + c.width && point.y > c.y && point.y < c.y + c.height;
            });
            if (comp) {
                this.setSelectedComponent(comp);
            } else {
                this.setSelectedComponent(undefined);
            }
        }
        this.state = 'done';
        this.lastMousePosition = CanvasRenderer.MOUSE_INITIAL_POSITION;
    }

    public handleMouseDown(e: any): void {
        if (this.state === 'saving') return;
        this.state = 'mouse_down';
        const point = this.eventInputPosition(e);
        this.lastMousePosition = point;
        const c = this.selectedComponent;
        if (c) {
            if (c.component_type === 'status_legend') return; // cannot resize
            if (c.pointOverlapsComponentCorner(point)) {
                this.state = 'resizing';
            }
        }
    }

    public handleMouseLeave(): void {
        if (this.state === 'saving') return;
        this.state = 'done';
    }

    public handleKeyDown(e: any): void {
        if (e.keyCode === 46) {
            this.deleteSelectedComponent();
        }
    }
}
